/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import org.junit.Assert;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Simulation eines Lve System Datenendgeraetes
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class LveDeKanal255Sim extends LveDeSim {

	/**
	 * Datenbeschreibung für Abrufe von LVE-LZD
	 */
	private static DataDescription ddLveAbruf;

	/**
	 * Ob man angemeldet ist fuer Abruf Telegramme
	 */
	private boolean anmeldetAbrufPuffer;
	/**
	 * Liste aller zuhoerigen DEs "broadcast Domene"
	 */
	private final Collection<LveDeSim> listDes;

	/**
	 * Standardkonstruktor
	 *
	 * @param dePid
	 *            Pid des DEs
	 * @param fsPid
	 *            Pid des FS
	 * @param listDes
	 *            Liste der DEs die in der "broadcast Domene" liegen
	 */
	public LveDeKanal255Sim(final String dePid, final String fsPid, final Collection<LveDeSim> listDes) {
		super(dePid, fsPid, 1);
		this.listDes = listDes;
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		super.initialize(connection);
		LveDeKanal255Sim.ddLveAbruf = erzeugeDatenBeschreibung(AbfrPufferKonstanten.ATG_TLS_LVEABRUF,
				AbfrPufferKonstanten.ASP_TLS_ABRUF, (short) 0);
		DeSim.getDav().subscribeReceiver(this, DeSim.getDav().getDataModel().getObject(getDePid()),
				LveDeKanal255Sim.ddLveAbruf, ReceiveOptions.normal(), ReceiverRole.receiver());
		anmeldetAbrufPuffer = true;
	}

	@Override
	public void update(final ResultData[] results) {
		super.update(results);
		for (final ResultData resDatei : results) {
			final Data data = resDatei.getData();

			if ((data != null) && resDatei.getDataDescription().getAttributeGroup().getPid()
					.equals(AbfrPufferKonstanten.ATG_TLS_LVEABRUF)) {
				final long zeit = data.getTimeValue(AbfrPufferKonstanten.ZEITPUNKT).getMillis();
				final int anzahl = data.getUnscaledValue(AbfrPufferKonstanten.ATR_ANZAHL).intValue();

				long aeltesteAbruf = AbfrPufferKonstanten.ZEIT_BEGINN;
				long kleinsteIntervall = AbfrPufferKonstanten.INTERVALL_MAX;

				for (final LveDeSim de : listDes) {
					if (de.isOnline() && (de.getLetzterZeitStempellGeschickt() < aeltesteAbruf)) {
						aeltesteAbruf = de.getLetzterZeitStempellGeschickt();
					} else if (de.isOnline() && (de.getErzeugungIntervall() < kleinsteIntervall)) {
						kleinsteIntervall = de.getErzeugungIntervall();
					}
				}

				Assert.assertTrue("SysDe: Abruf fur Zeitpunkt das schon abgerufen ist", aeltesteAbruf < zeit);
				Assert.assertTrue("SysDe: Anzahl der DatenSaetze zu hoch",
						((ZeitSimulator.getTimeMillis() - zeit) / kleinsteIntervall) >= anzahl);

				if (isAnfrageNichtVerstanden()) {
					sendeNegativeQuittung();
				} else {
					for (final LveDeSim de : listDes) {

						de.sendeLZD(zeit, anzahl);
					}
				}
			}
		}
	}

	@Override
	public void unsubscribe() {
		if (anmeldetAbrufPuffer) {
			DeSim.getDav().unsubscribeReceiver(this, DeSim.getDav().getDataModel().getObject(getDePid()),
					LveDeKanal255Sim.ddLveAbruf);
		}
		super.unsubscribe();
	}

}
