/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.abfrpuffer.SystemDatenEndgeraet.SystemDeStatus;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Besorgt, dass die Langzeitdaten regelmaesig abgefragt werden.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class LveDeVerwaltung extends AbstraktDeVerwaltung {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Tabelle mit Assoziationen von Fahrstreifen SystemObjekten zum
	 * Datenengeraeten.
	 */
	private final Map<String, String> mapFSzumDE;
	/**
	 * Liste aller Fahrstreifen, auf deren Daten wir angemeldet sind.
	 */
	private final Collection<SystemObject> archivierteFahrStreifen;
	/**
	 * Liste aller Fahrstreifen, auf die wir uns aufmelden sollen.
	 */
	private final Collection<SystemObject> zurAnmeldungFahrStreifen;
	/**
	 * Liste aller Fahrstreifen, die wir abmelden sollen.
	 */
	private final Collection<SystemObject> zurAbmeldungFahrStreifen;
	/**
	 * Datenbeschreibung für LVE-LZD Empfang.
	 */
	private static DataDescription ddLveLzdEmpfang;
	/**
	 * Datenbeschreibung für Abrufe von LVE-LZD senden.
	 */
	private static DataDescription ddLveAbrufSend;
	/**
	 * Datenbeschreibung von LVE-DE Betriebsparameter.
	 */
	private static DataDescription ddLveParam;
	/**
	 * Datenbeschreibung von LVE-DE Negative Quittung.
	 */
	private static DataDescription ddLveNegQuit;
	/**
	 * Ob der Empfang und Versand Telegramme zu DEs angemeldet ist.
	 */
	private boolean deEmfangVersandAngemeldet;

	/**
	 * Standardkonstruktor.
	 *
	 * @param abfHaupt
	 *            Hauptlklasse des Moduls
	 * @param steuerModule
	 *            Liste der verwateten Steuermodule
	 */
	public LveDeVerwaltung(final AbfragePufferdaten abfHaupt, final Collection<Station> steuerModule) {

		super(abfHaupt, steuerModule);
		mapFSzumDE = new Hashtable<>();
		archivierteFahrStreifen = new LinkedList<>();
		zurAbmeldungFahrStreifen = new LinkedList<>();
		zurAnmeldungFahrStreifen = new LinkedList<>();

		LveDeVerwaltung.ddLveParam = getVerwaltungHaupt().erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_LVEBETRIEBSPARAM, AbfrPufferKonstanten.ASP_PARAMETERSOLL,
				DataDescription.NO_SIMULATION_VARIANT_SET);
		LveDeVerwaltung.ddLveAbrufSend = getVerwaltungHaupt().erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_LVEABRUF, AbfrPufferKonstanten.ASP_TLS_ABRUF,
				DataDescription.NO_SIMULATION_VARIANT_SET);
		LveDeVerwaltung.ddLveLzdEmpfang = getVerwaltungHaupt().erzeugeDatenBeschreibung(DUAKonstanten.ATG_LZD,
				DUAKonstanten.ASP_EXTERNE_ERFASSUNG, DataDescription.NO_SIMULATION_VARIANT_SET);
		LveDeVerwaltung.ddLveNegQuit = getVerwaltungHaupt().erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_LVENEGATIVE_QUITTUNG, AbfrPufferKonstanten.ASP_TLS_ANTWORT,
				DataDescription.NO_SIMULATION_VARIANT_SET);

		Station.setLveDeVerwaltung(this);
	}

	/**
	 * Erste Anmeldung fuer Datenempfang und senden.
	 */
	@Override
	public void ersteAnmeldung() {
		Collection<SystemObject> lveDEs;

		lveDEs = Station.getAlleLveDEs(getSteuerModule());
		getDav().subscribeReceiver(getVerwaltungHaupt(), lveDEs, LveDeVerwaltung.ddLveParam, ReceiveOptions.normal(),
				ReceiverRole.receiver());
		getDav().subscribeReceiver(getVerwaltungHaupt(), lveDEs, LveDeVerwaltung.ddLveNegQuit, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		lveDEs = Station.getAlleLveSystemDEs(getSteuerModule());
		try {
			getDav().subscribeSender(getVerwaltungHaupt(), lveDEs, LveDeVerwaltung.ddLveAbrufSend, SenderRole.sender());
			deEmfangVersandAngemeldet = true;
		} catch (final OneSubscriptionPerSendData e) {
			LveDeVerwaltung.LOGGER.error("Anmeldung für LveAbrufe Sender unmöglich:" + e.getMessage());
			getVerwaltungHaupt().getnachrichtenSender().sendMessage(MessageType.SYSTEM_DOMAIN, MessageGrade.FATAL,
					"Anmeldung für LveAbrufe Sender unmöglich:" + e.getMessage());
		}
	}

	/**
	 * Anmeldung fuer Daten Empfang und Sendung, wird bei jedem
	 * Archviparametrierungveraenderung gerufen.
	 */
	@Override
	public void anmeldung() {

		streamBasierteArchivAbfrage();
		getDav().subscribeReceiver(getVerwaltungHaupt(), zurAnmeldungFahrStreifen, LveDeVerwaltung.ddLveLzdEmpfang,
				ReceiveOptions.delayed(), ReceiverRole.receiver());
		getDav().unsubscribeReceiver(getVerwaltungHaupt(), zurAbmeldungFahrStreifen, LveDeVerwaltung.ddLveLzdEmpfang);

		archivierteFahrStreifen.addAll(zurAnmeldungFahrStreifen);
		archivierteFahrStreifen.removeAll(zurAbmeldungFahrStreifen);

		for (final SystemObject fs : zurAbmeldungFahrStreifen) {
			final DatenEndgeraet de = Station.getDatenEndgeraet(mapFSzumDE.get(fs.getPid()));
			if (de != null) {
				final Station st = de.getStation();
				aktualisiereStatus(st, "");
			}
		}

		zurAbmeldungFahrStreifen.clear();
		zurAnmeldungFahrStreifen.clear();
	}

	/**
	 * Aktualisiert den Status der Station, nach Ankunft von Betirebsmeldungen,
	 * FehlerMeldungen, oder ArchivDaten.
	 *
	 * @param st
	 *            Station
	 * @param s
	 *            Beschreibung fuer die Statusausgabe
	 */
	@Override
	public void aktualisiereStatus(final Station st, final String s) {
		final SystemDeStatus status = st.getSystemLveDe().getStatus();
		if (st.istLveOnline()) {
			if (st.istLveParametriert()) {
				st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
				statusAktualisierungAusgabe(st, s);
				generiereAbfrage(st);
			} else {
				st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_NICHT_PARAMETRIERT);
				if (status != st.getSystemLveDe().getStatus()) {
					statusAktualisierungAusgabe(st, s);
				}
			}
		} else {
			st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_NICHT_VERBUNDEN);
			if (status != st.getSystemLveDe().getStatus()) {
				statusAktualisierungAusgabe(st, s);
			}
		}
	}

	/**
	 * Abmeldung fuer Daten Empfang und Sendung.
	 */
	@Override
	public void abmeldung() {
		getDav().unsubscribeReceiver(getVerwaltungHaupt(), archivierteFahrStreifen, LveDeVerwaltung.ddLveLzdEmpfang);
		Collection<SystemObject> lveDEs;

		if (deEmfangVersandAngemeldet) {
			lveDEs = Station.getAlleLveSystemDEs(getSteuerModule());
			getDav().unsubscribeSender(getVerwaltungHaupt(), lveDEs, LveDeVerwaltung.ddLveAbrufSend);
			lveDEs = Station.getAlleLveDEs(getSteuerModule());
			getDav().unsubscribeReceiver(getVerwaltungHaupt(), lveDEs, LveDeVerwaltung.ddLveParam);
			deEmfangVersandAngemeldet = false;
		}
	}

	/**
	 * Aktualisiert dem Zeitstempell wenn Datei FG1 LZD vom Fahrstreifen
	 * angekommen sind.
	 *
	 * @param datei
	 *            Die empfangene Datei
	 */
	@Override
	public void aktualisiereDeZeitStemepel(final ResultData datei) {
		final SystemObject fs = datei.getObject();
		final Data daten = datei.getData();
		final DataState ds = datei.getDataState();
		long zeit = 0;
		if (daten == null) {
			// nichts zu tun
			return;
		} else if (ds.equals(DataState.DATA)) {
			zeit = datei.getDataTime();
			final String pidDe = mapFSzumDE.get((fs.getPid()));

			final DatenEndgeraet de = Station.getDatenEndgeraet(pidDe);
			final Station st = de.getStation();

			final long anzahl = de.decrementAnzahlErwarteteDatenSaetze();
			de.updateLetzterDatensatzZeitStempel(zeit);

			if (anzahl < 0) {
				LveDeVerwaltung.LOGGER.warning(
						"Unerwartetes Lve Datum von " + de.getSoDE().getPid() + " eingekommen, fehlende DS:" + anzahl);
			}
			final long jetzt = currentTimeMillis();

			final boolean allesGekommen = st.sindAlleLveDatenSaetzeGekommen(jetzt);

			if (allesGekommen && (st.getSystemLveDe().getStatus() == SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT)) {
				if ((st.getLveMaxAnzahlFehlendenDatenSaetzen(jetzt) > 0) && !st.antwortLveNegQuittBekommen()) {
					st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
					generiereAbfrage(st);
				} else {
					st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
					st.lveTimeoutAbbrechen();
					abfrageEnde(st, false);
				}
			}
		}
	}

	@Override
	public void bearbeiteNegativeQuittung(final ResultData datei) {
		final SystemObject lveDe = datei.getObject();
		final DatenEndgeraet de = Station.getDatenEndgeraet(lveDe.getPid());
		if (de instanceof SystemDatenEndgeraet) {
			final SystemDatenEndgeraet sysDe = (SystemDatenEndgeraet) de;
			sysDe.erwarteKeineAntwort(true);
			sysDe.getStation().lveTimeoutAbbrechen();
		} else if (de != null) {
			// Negative Quittung von einzelnen DE
			de.setAnzahlErwarteteDatenSaetze(0);
			de.setAntwortetNegQuitt(true);

			final Station st = de.getStation();
			final long jetzt = currentTimeMillis();
			final boolean allesGekommen = st.sindAlleLveDatenSaetzeGekommen(jetzt);

			// Wenn von anderen DEs schon alles angekommen ist
			if (allesGekommen && (st.getSystemLveDe().getStatus() == SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT)) {
				st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
				st.lveTimeoutAbbrechen();
			}
		}
	}

	/**
	 * Liest die Betriebsparameter eines LveDe aus, aus der empfangenen Daten.
	 *
	 * @param datei
	 *            empfangene Datei
	 */
	@Override
	public void setBetriebsParameter(final ResultData datei) {
		final SystemObject de = datei.getObject();
		final Data daten = datei.getData();
		if (daten != null) {
			final int zeitCode = daten.getUnscaledValue(AbfrPufferKonstanten.ATR_INTERVALLDAUER_LZ).intValue();
			final long zeit = AbfrPufferKonstanten.convertLveZeitCodeZurZeit(zeitCode);

			final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(de.getPid());
			final Station st = datenEndgeraet.getStation();

			final boolean warParametriert = st.istLveParametriert();
			datenEndgeraet.setErzeugungsIntervall(zeit);
			final boolean istParametriert = st.istLveParametriert();

			if (warParametriert != istParametriert) {
				aktualisiereStatus(st, "Betriebsparam eingekommen ");
			}
		}
	}

	/**
	 * Generiert eine LZD Abfrage .
	 *
	 * @param st
	 *            Die Station fuer die die Abfrage geruft werden soll
	 */
	@Override
	public void generiereAbfrage(final Station st) {

		if (st.getSystemLveDe() == null) {
			return;
		}
		// eine Sekunde Mehr, damit wir nicht die selben Daten nochmal abfragen
		final long zeitStempelLetzterDaten = st.getAeltesteLveZeitStempel() + (1000L * 60L * 60L);
		final long jetzt = currentTimeMillis();
		long anzahl = st.getLveMaxAnzahlFehlendenDatenSaetzen(jetzt);

		if (!st.istLveParametriert() || !st.istLveOnline()) {
			LveDeVerwaltung.LOGGER.error("Daten Abfrage nicht moeglich : Parametriert:" + st.istLveParametriert()
			+ " Online:" + st.istLveOnline() + " Status = " + st.getSystemLveDe().getStatus());
			return;
		} else if (st.getSystemLveDe().getStatus() != SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH) {
			LveDeVerwaltung.LOGGER.error("Daten Abfrage nicht moeglich : Fehlende DatenSaetze = " + anzahl
					+ " Status = " + st.getSystemLveDe().getStatus());
			return;
		} else if (st.lveFlackert(jetzt)) {
			LveDeVerwaltung.LOGGER.warning(
					"Lve Station " + st.getPid() + " flackert, Abfragen um " + new Date(jetzt) + " nicht erlaubt.");
			return;
		} else if ((anzahl == 0)
				|| (getVerwaltungHaupt().getAbrufIntervallLve() < st.getLveLZDErzeugungsIntervallMinimum())) {
			LveDeVerwaltung.LOGGER.warning("Lve Abfrage nicht noetig, daten aktuell: AbrufINtervall"
					+ getVerwaltungHaupt().getAbrufIntervallLve() + " Erzuegungintervall: "
					+ st.getLveLZDErzeugungsIntervallMinimum() + " Zeit" + new Date(jetzt));
			return;
		}
		// maximaler Datensatzanzahl begrentzt
		if (anzahl > AbfrPufferKonstanten.MAX_DATENSAETZE) {
			anzahl = AbfrPufferKonstanten.MAX_DATENSAETZE;
		}

		st.setErwarteLveDatensaetze(anzahl);
		final long timeout = 5000L + (st.getLveDeListe().size() * anzahl * AbfrPufferKonstanten.BASIS_TIMEOUT_LVE);

		generiereAbfrage(st.getSystemLveDe().getSoDE(), zeitStempelLetzterDaten, anzahl);
		st.getSystemLveDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT);
		st.getSystemLveDe().setLetzterAbruf(zeitStempelLetzterDaten);
		st.setLveTimeoutWecker(timeout);

		abfrageAusgabe(st, anzahl, jetzt, timeout);
	}

	/**
	 * Generiert eine LZD Abfrage.
	 *
	 * @param systemDe
	 *            Das De mit Kanal 255, dem man abfragen soll
	 * @param zeitStempel
	 *            Zeit, aus der wir die Daten benoetigen
	 * @param anzahl
	 *            Anzahl der benoetigten Datensaetzen
	 */
	@Override
	public void generiereAbfrage(final SystemObject systemDe, final long zeitStempel, final long anzahl) {
		final Data data = getDav()
				.createData(getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_TLS_LVEABRUF));
		final Data zeitData = data.getItem(AbfrPufferKonstanten.ZEITPUNKT);
		zeitData.asTimeValue().setMillis(zeitStempel);
		final Data anzahlData = data.getItem(AbfrPufferKonstanten.ATR_ANZAHL);
		anzahlData.asUnscaledValue().set(anzahl);
		final long zeit = currentTimeMillis();
		final ResultData rd = new ResultData(systemDe, LveDeVerwaltung.ddLveAbrufSend, zeit, data);

		try {
			getDav().sendData(rd);
			// System.out.println("LVE-Abruf: " + rd);
		} catch (final DataNotSubscribedException e) {
			LveDeVerwaltung.LOGGER.error(e.getMessage());
			e.printStackTrace();
		} catch (final SendSubscriptionNotConfirmed e) {
			LveDeVerwaltung.LOGGER.error(e.getMessage());
			e.printStackTrace();
		}
	}

	@Override
	public void bearbeiteNeueEinstellung(final DataIdentification dateiID, final Data neueDatei) {

		if (!(dateiID.getObject().isOfType(DUAKonstanten.TYP_FAHRSTREIFEN)
				&& dateiID.getDataDescription().getAttributeGroup().getPid().equals(DUAKonstanten.ATG_LZD)
				&& dateiID.getDataDescription().getAspect().getPid().equals(DUAKonstanten.ASP_EXTERNE_ERFASSUNG))) {
			return;
		}

		boolean archiviert = true;

		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			archiviert = false;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_FAHRSTREIFEN),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		if (confDaten != null) {
			if ((confDaten.getItem(AbfrPufferKonstanten.ATR_FAHRSTREIFENQUELLE).asReferenceValue() != null)
					&& (confDaten.getItem(AbfrPufferKonstanten.ATR_FAHRSTREIFENQUELLE).asReferenceValue() != null)) {
				final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_FAHRSTREIFENQUELLE).asReferenceValue()
						.getValueText();

				// FS assozieiert mit DEs die nicht verwaltet werden
				// interessieren
				// uns
				// nicht
				if (!Station.enthaeltEndgeraet(dePid)) {
					return;
				}

				if (!mapFSzumDE.containsKey(dateiID.getObject().getPid())) {
					mapFSzumDE.put(dateiID.getObject().getPid(), dePid);
				}

				zurAnmeldungFahrStreifen.add(dateiID.getObject());

				final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);

				datenEndgeraet.setArchiviert(archiviert);
				datenEndgeraet.setSoAssoziiert(dateiID.getObject());
			}
		}
	}

	@Override
	public void bearbeiteGeloeschteEinstellung(final DataIdentification dateiID, final Data alteDatei) {

		if (!(dateiID.getObject().isOfType(DUAKonstanten.TYP_FAHRSTREIFEN)
				&& dateiID.getDataDescription().getAttributeGroup().getPid().equals(DUAKonstanten.ATG_LZD)
				&& dateiID.getDataDescription().getAspect().getPid().equals(DUAKonstanten.ASP_EXTERNE_ERFASSUNG))) {
			return;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_FAHRSTREIFEN),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_FAHRSTREIFENQUELLE).asReferenceValue()
				.getValueText();

		// FS assozieiert mit DEs die nicht verwaltet werden interessieren uns
		// nicht
		if (!Station.enthaeltEndgeraet(dePid)) {
			return;
		}

		if (!mapFSzumDE.containsKey(dateiID.getObject().getPid())) {
			mapFSzumDE.put(dateiID.getObject().getPid(), dePid);
		}

		zurAbmeldungFahrStreifen.add(dateiID.getObject());

		final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);
		datenEndgeraet.setArchiviert(false);
	}

	@Override
	public void bearbeiteVeraenderteEinstellung(final DataIdentification dateiID, final Data alteDatei,
			final Data neueDatei) {

		if (!(dateiID.getObject().isOfType(DUAKonstanten.TYP_FAHRSTREIFEN)
				&& dateiID.getDataDescription().getAttributeGroup().getPid().equals(DUAKonstanten.ATG_LZD)
				&& dateiID.getDataDescription().getAspect().getPid().equals(DUAKonstanten.ASP_EXTERNE_ERFASSUNG))) {
			return;
		}

		boolean neuArchiviert = true;
		boolean altArchiviert = true;

		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			neuArchiviert = false;
		}
		if (alteDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			altArchiviert = false;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_FAHRSTREIFEN),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		if (confDaten == null) {
			return;
		}
		final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_FAHRSTREIFENQUELLE).asReferenceValue()
				.getValueText();

		// FS assozieiert mit DEs die nicht verwaltet werden interessieren uns
		// nicht
		if (!Station.enthaeltEndgeraet(dePid)) {
			return;
		}

		if (!mapFSzumDE.containsKey(dateiID.getObject().getPid())) {
			mapFSzumDE.put(dateiID.getObject().getPid(), dePid);
		}

		final DatenEndgeraet de = Station.getDatenEndgeraet(dePid);
		if (altArchiviert != de.isArchiviert()) {
			LveDeVerwaltung.LOGGER.error("Archivierung status nicht korrekt " + de.getSoDE().getPid());
		}

		if (neuArchiviert != altArchiviert) {
			if (neuArchiviert) {
				zurAnmeldungFahrStreifen.add(dateiID.getObject());
			} else {
				zurAbmeldungFahrStreifen.add(dateiID.getObject());
			}

			final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);
			datenEndgeraet.setArchiviert(neuArchiviert);

		}
	}

	/**
	 * Sendet eine StreamAbfrage an das Archivsystem.
	 */
	@Override
	public void streamBasierteArchivAbfrage() {
		final DataDescription datenBeschreibung = getVerwaltungHaupt().erzeugeDatenBeschreibung(DUAKonstanten.ATG_LZD,
				DUAKonstanten.ASP_EXTERNE_ERFASSUNG, DataDescription.NO_SIMULATION_VARIANT_SET);
		final long jetzt = currentTimeMillis();

		// ///////////////////////////
		getVerwaltungHaupt().streamAbfrage(this, zurAnmeldungFahrStreifen, datenBeschreibung, true, 1, jetzt);
	}

	@Override
	public void bearbeiteArchivDaten(final SystemObject objekt, final DataDescription datenBeschreibung,
			final DataState status, final Data daten, final long zeitStempel) {

		final String depid = mapFSzumDE.get(objekt.getPid());
		final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(depid);
		if (datenEndgeraet != null) {
			if (status == DataState.END_OF_ARCHIVE) {
				if (datenEndgeraet.isNochNieDaten()) {
					datenEndgeraet.updateLetzterDatensatzZeitStempel(AbfrPufferKonstanten.ZEIT_BEGINN);
					datenEndgeraet.setArchivAbgefragt(true);
					final Station st = datenEndgeraet.getStation();
					aktualisiereStatus(st, "Archiv Parametrierung");
					return;
				}
				final boolean warArchivAngefragt = datenEndgeraet.isArchivAbgefragt();
				datenEndgeraet.setArchivAbgefragt(true);
				final Station st = datenEndgeraet.getStation();
				if (!warArchivAngefragt) {
					aktualisiereStatus(st, "Archiv Parametrierung");
				}
			} else {
				datenEndgeraet.updateLetzterDatensatzZeitStempel(zeitStempel);
			}
		}
	}

}
